///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAttrCond.cc
// ---------------
// Cego database table field implementation
//     
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoAttrCond
// 
// Description: 
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include "CegoAttrCond.h"

#include <stdlib.h>
#include <string.h>

CegoAttrCond::CegoAttrCond()
{
}

CegoAttrCond::~CegoAttrCond()
{
}

void CegoAttrCond::add(const CegoAttrComp& attrComp)
{
    _attrCompList.Insert(attrComp);
}

int CegoAttrCond::numComp() const
{
    return _attrCompList.Size();
}

ListT<CegoAttrComp>& CegoAttrCond::getAttrCompList()
{
    return _attrCompList;
}

bool CegoAttrCond::setup(const ListT<CegoField>& fl)
{
    CegoAttrComp *pAC = _attrCompList.First();
    while ( pAC )
    {
	if ( pAC->setup(fl) == false )
	    return false;
	pAC = _attrCompList.Next();
    }
    return true;
}

bool CegoAttrCond::setup(ListT<CegoField>* joinBuf, int offset, int size)
{
    CegoAttrComp *pAC = _attrCompList.First();
    while ( pAC )
    {
	if ( pAC->setup(joinBuf, offset, size) == false )
	    return false;
	pAC = _attrCompList.Next();
    }
    return true;
}

void CegoAttrCond::asConjunctionList(ListT<CegoPredDesc*>& conjunctionList) const
{
    CegoAttrComp *pAC = _attrCompList.First();
    while ( pAC )
    {
	CegoPredDesc *pP;
	if ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() == false )   
	{	   
	    pP = new CegoPredDesc(new CegoExpr(new CegoTerm(new CegoFactor( new CegoAttrDesc(pAC->getAttrName())))),
				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getAttrDesc().clone()))),
				  pAC->getComparison());
	}
	else if ( pAC->getCompMode() == CegoAttrComp::VAL || 
		  ( pAC->getCompMode() == CegoAttrComp::ATTR && pAC->isSetup() ) )
	{
	    pP = new CegoPredDesc(new CegoExpr(new CegoTerm(new CegoFactor( new CegoAttrDesc(pAC->getAttrName())))),
				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
				  pAC->getComparison());
	}
	else if ( pAC->getCompMode() == CegoAttrComp::BTWN )	     
	{
	    pP = new CegoPredDesc(new CegoExpr(new CegoTerm(new CegoFactor( new CegoAttrDesc(pAC->getAttrName())))),
				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue()))),
				  new CegoExpr(new CegoTerm(new CegoFactor( pAC->getFieldValue2()))));
	}

	conjunctionList.Insert(pP);
	
	pAC = _attrCompList.Next();
    }	
}

CegoAttrCond::IndexMatch CegoAttrCond::checkIndex(const ListT<CegoField>& schema) const
{

    CegoField *pF = schema.First();
    while ( pF )
    {
	if ( _attrCompList.Find(CegoAttrComp(pF->getAttrName())) == 0)
	{ 
	    return INAPP;
	}
	pF = schema.Next();
    }

    CegoAttrComp *pComp = _attrCompList.First();
    while ( pComp )
    {
	if ( schema.Find(CegoField(Chain(), pComp->getAttrName())) == 0)
	{
	    return PART;
	}
	pComp = _attrCompList.Next();
    }
    
    return FULL;


}
    
void CegoAttrCond::setIdxSchema(ListT<CegoField>& schema)
{
    _idxSchema = schema;
}

void CegoAttrCond::setPrimaryComparison(CegoComparison comp)
{
    CegoAttrComp* pAC = _attrCompList.First();
    if ( pAC )
    {
	pAC->setComparison(comp);    
    }
    else
    {
	throw Exception(EXLOC, "Cannot set primary comparison");
    }
}

CegoAttrComp::CompMode CegoAttrCond::getPrimaryCompMode() const
{
    CegoAttrComp* pAC = _attrCompList.First();
    if ( pAC )
    {
	return pAC->getCompMode();
    }
    else
    {
	throw Exception(EXLOC, "Cannot get primary comp mode");
    }
    
}

CegoComparison CegoAttrCond::getPrimaryComparison() const
{
    CegoAttrComp* pAC = _attrCompList.First();
    if ( pAC )
    {
	return pAC->getComparison();
    }
    else
    {
	throw Exception(EXLOC, "Cannot get primary comparison");
    }
}

CegoAttrCond& CegoAttrCond::operator = ( const CegoAttrCond& ac)
{
    _idxSchema = ac._idxSchema;
    _attrCompList = ac._attrCompList;
    return (*this);
}

bool CegoAttrCond::operator == ( const CegoAttrCond& ac) const
{
    
    CegoAttrComp* pAC1 = ac._attrCompList.First();
    CegoAttrComp* pAC2 = _attrCompList.First();
    while ( pAC1 && pAC2 )
    {
	if ( ( *pAC1 == *pAC2 ) == false)
	    return false;

	pAC1 = ac._attrCompList.Next();
	pAC2 = _attrCompList.Next();
    }
    if ( pAC1 || pAC2 )
	return false;

    return true;
    
}

bool CegoAttrCond::operator != ( const CegoAttrCond& ac) const
{
    if ( *this == ac )
	return false;
    return true;
}

Chain CegoAttrCond::toChain() const
{
    Chain s;

    CegoAttrComp *pAC = _attrCompList.First();
    while ( pAC )
    {	
	s += pAC->toChain();
	pAC = _attrCompList.Next();
	if ( pAC )
	    s += Chain(" and ");
    }
    return s;

}

ostream& operator << (ostream& s, const CegoAttrCond& ac)
{
    s << ac.toChain();
    return s;
}

    

